@@ -6,6 +6,7 @@ use libp2p::swarm::{
 };
 use libp2p::{Multiaddr, PeerId};
 use std::collections::VecDeque;
+use std::fmt;
 use std::sync::Arc;
 use std::task::{Context, Poll};
 
@@ -19,14 +20,26 @@ pub enum BroadcastEvent {
     Unsubscribed(PeerId, Topic),
     Received(PeerId, Topic, Arc<[u8]>),
 }
+type Handler = OneShotHandler<BroadcastConfig, Message, HandlerEvent>;
 
-#[derive(Debug, Default)]
+#[derive(Default)]
 pub struct Broadcast {
     config: BroadcastConfig,
     subscriptions: FnvHashSet<Topic>,
     peers: FnvHashMap<PeerId, FnvHashSet<Topic>>,
     topics: FnvHashMap<Topic, FnvHashSet<PeerId>>,
-    events: VecDeque<NetworkBehaviourAction<Message, BroadcastEvent>>,
+    events: VecDeque<NetworkBehaviourAction<BroadcastEvent, Handler>>,
+}
+
+impl fmt::Debug for Broadcast {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Broadcast")
+            .field("config", &self.config)
+            .field("subscriptions", &self.subscriptions)
+            .field("peers", &self.peers)
+            .field("topics", &self.topics)
+            .finish()
+    }
 }
 
 impl Broadcast {
@@ -37,6 +50,18 @@ impl Broadcast {
         }
     }
 
+    pub fn subscribed(&self) -> impl Iterator<Item = &Topic> + '_ {
+        self.subscriptions.iter()
+    }
+
+    pub fn peers(&self, topic: &Topic) -> Option<impl Iterator<Item = &PeerId> + '_> {
+        self.topics.get(topic).map(|peers| peers.iter())
+    }
+
+    pub fn topics(&self, peer: &PeerId) -> Option<impl Iterator<Item = &Topic> + '_> {
+        self.peers.get(peer).map(|topics| topics.iter())
+    }
+
     pub fn subscribe(&mut self, topic: Topic) {
         self.subscriptions.insert(topic);
         let msg = Message::Subscribe(topic);
@@ -144,7 +169,7 @@ impl NetworkBehaviour for Broadcast {
         &mut self,
         _: &mut Context,
         _: &mut impl PollParameters,
-    ) -> Poll<NetworkBehaviourAction<Message, Self::OutEvent>> {
+    ) -> Poll<NetworkBehaviourAction<BroadcastEvent, Handler>> {
         if let Some(event) = self.events.pop_front() {
             Poll::Ready(event)
         } else {
